/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/*
 * GameGUI.java
 *
 * Created on 25-nov-2009, 15:00:14
 */
package biteme;

import java.awt.Dimension;
import java.awt.GridBagLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
import javax.swing.Timer;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import java.util.Calendar;
import java.util.Scanner;
//import javax.swing.JLabel;

/**
 * Class for the GUI of the game
 * @author s070693
 */
public class GameGUI extends javax.swing.JFrame {

    private boolean introMusicPlayed = false;
    private Timer pacmanTimer;
    private Timer ghostTimer;
    private Timer scoreTimer;
    private boolean canPacmanMove = false;
    private boolean paused = true;
    private String levelPlayed;
    private Calendar time;
    public int score;
    public int startTime;
    public int stopTime;
    HighScore newscore = new HighScore();
    ScoreList dayScores;
    ScoreList weekScores;
    ScoreList allScores;
    // Label grid array containing the components at every [x][y]
    //JLabel[][] grid = new JLabel[32][22];
    ImageIcon symbol = new ImageIcon();
    JFrame parent;

    /**
     * Creates new form GameGUI.
     * @param JFrame the frame
     */
    public GameGUI(JFrame pParent, String mazeFilePath) {
        parent = pParent;
        levelPlayed = mazeFilePath;
        initComponents();
        if (parent instanceof EditorGUI) {
            Main_menu_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/biteme/back.png"))); // NOI18N
        }

        //icon
        setIconImage(Toolkit.getDefaultToolkit().getImage(MainMenu.class.getResource("biteme_icon.png")));
        pacmanTimer = new Timer(100, pacmanLoop);
        ghostTimer = new Timer(300, ghostLoop);
        scoreTimer = new Timer(1000, scoreLoop);

        // set panel GamePanel as gridbaylayout
        this.GamePanel.setLayout(new GridBagLayout());

        this.GamePanel.validate();

        MazeIO.loadMaze(GamePanel, mazeFilePath);
        setTitle(MazeIO.getFileName(mazeFilePath));
        dayScores = MazeIO.loadDayScores(mazeFilePath);
        weekScores = MazeIO.loadWeekScores(mazeFilePath);
        allScores = MazeIO.loadAllTimeScores(mazeFilePath);

        GamePanel.initialDraw();

        // Request focus; anders krijgt de maze geen keypress binnen
        // DIT MOET ELKE KEER AANGEROEPEN WORDEN NADAT EEN ANDERE CONTROL DE FOCUS KRIJGT
        // (Buttons, etc)
        this.GamePanel.requestFocus();

        Dimension minSize = new Dimension(980, 900);
        this.setMinimumSize(minSize);
        this.setPreferredSize(minSize);
        this.validate();
    }

    private void pause() {
        this.pause(true);
    }

    private void pause(boolean playIntermissionSound) {
        if (playIntermissionSound) {
            Sound.playPause();
        }
        paused = true;
        canPacmanMove = false;
        ghostTimer.stop();
        scoreTimer.stop();
        time = Calendar.getInstance();
        stopTime = (time.get(Calendar.HOUR_OF_DAY) * 3600) + (time.get(Calendar.MINUTE) * 60) + time.get(Calendar.SECOND);
        score = score + (stopTime - startTime);
        ScoreButton.setText(Messages.GAME_SCORE1 + score + Messages.GAME_SCORE2);
        // TODO: store current playing time for highscore
    }

    private void unPause() {
        if (!introMusicPlayed) {
            Sound.playIntro();
            introMusicPlayed = true;
        } else {
           // Sound.stopPause();
        }
        paused = false;
        canPacmanMove = true;
        ghostTimer.start();
        scoreTimer.start();
        time = Calendar.getInstance();
        startTime = (time.get(Calendar.HOUR_OF_DAY) * 3600) + (time.get(Calendar.MINUTE) * 60) + time.get(Calendar.SECOND);
    }


    ActionListener pacmanLoop = new ActionListener() {

        public void actionPerformed(ActionEvent e) {
            // 100 milliseconds have passed

            canPacmanMove = true;
            pacmanTimer.stop();
        }
    };
    ActionListener ghostLoop = new ActionListener() {

        public void actionPerformed(ActionEvent e) {
            GamePanel.executeDijkstra();
            for (SmartGhost g : GamePanel.getSmartGhosts()) {
                g.move();
                checkDeath();
            }
            for (StupidGhost g : GamePanel.getStupidGhosts()) {
                g.move();
                checkDeath();
            }
        }
    };
   ActionListener scoreLoop = new ActionListener() {

        public void actionPerformed(ActionEvent e) {
            updateScore();
        }
    };

    private void updateScore() {
            time = Calendar.getInstance();
            stopTime = (time.get(Calendar.HOUR_OF_DAY) * 3600) + (time.get(Calendar.MINUTE) * 60) + time.get(Calendar.SECOND);
            score = score + (stopTime - startTime);
            startTime = stopTime;
            ScoreButton.setText(Messages.GAME_SCORE1 + score + Messages.GAME_SCORE2);
    }

    private void checkDeath() {
        if (GamePanel.getPacman().getTile().containsGhost()) {
            if (!paused) {
                Sound.playPacmanDying();
                pause(false);
                if (parent instanceof EditorGUI) {
                    if (Popup.ok(Messages.GAME_POPUP_GAME_OVER) == 0) {
                        GameGUI gameGUI = new GameGUI(parent, levelPlayed);
                        gameGUI.setVisible(true);
                        this.setVisible(false);
                        dispose();
                    }
                } else {
                    if (Popup.retrymain(Messages.GAME_POPUP_GAME_OVER) == 1) {
                        GameGUI gameGUI = new GameGUI(parent, levelPlayed);
                        gameGUI.setVisible(true);
                        this.setVisible(false);
                        dispose();
                    } else {
                        LevelSelect levelselect = new LevelSelect(parent);
                        levelselect.setVisible(true);
                        this.setVisible(false);
                        dispose();
                    }
                }
            }
        }
    }

    private void checkWin() {
        int i;
        if (GamePanel.getCoinCount() == 0 || (GamePanel.getCoinCount() == 1 && GamePanel.getPacman().getTile().getStatus().containsCoin())) {
            Sound.playWin();
            pause(false);
            if (parent instanceof EditorGUI) {
                System.out.println(Popup.ok(Messages.GAME_POPUP_TEST_COMPLETE));
                MazeIO.makePlayable(levelPlayed);
                // naar editor of naar het menu terug?
                MainMenu mainmenu = new MainMenu(null);
                mainmenu.setVisible(true);
                setVisible(false);
                dispose();
            } else {
                // Only ask for a name if the score should be saved:
                if(score<dayScores.getScore(9).getScore()) {
                //stopTime = (time.get(Calendar.HOUR_OF_DAY) * 3600) + (time.get(Calendar.MINUTE) * 60) + time.get(Calendar.SECOND);
               // score = score + (stopTime - startTime);
                //String str = Messages.POPUP_GAME_COMPLETE;
                String str = Messages.GAME_POPUP_COMPLETE1 + score + Messages.GAME_POPUP_COMPLETE2;
                String defname;

                // get default name
                defname = DefaultName.getName();


                String name = Popup.savecancel(str,defname);
                if(name != null){
                    
                    while(name.length() > 15 || name.equals("Enter Name")){
                        if(name.length()>15){
                                str = Messages.GAME_POPUP_CHECK_LENGHT;
                                name = Popup.savecancel(str,name);
                        }else{
                                str = Messages.GAME_POPUP_NONAME;
                                name = Popup.savecancel(str,name);
                        }
                    }

                    // sla nieuwe default name op
                    DefaultName.setName((name));


                    for (i = 1; i < 10; i++) {
                        if (dayScores.getScore(i).getScore() > score) {
                            dayScores.editScore(10, name, score, time);
                        }
                        if (weekScores.getScore(i).getScore() > score) {
                            weekScores.editScore(10, name, score, time);
                        }
                        if (allScores.getScore(i).getScore() > score) {
                            allScores.editScore(10, name, score, time);
                        }

                    }
                }
            } else {
                    System.out.println(Popup.ok(Messages.GAME_POPUP_COMPLETE1 + score + Messages.GAME_POPUP_COMPLETE2));
            }

                dayScores.sortList();
                weekScores.sortList();
                allScores.sortList();
                //save the scores into the maze.
                Maze fakemaze = new Maze();
                MazeIO.loadMaze(fakemaze, levelPlayed);
                MazeIO.saveMaze(fakemaze, allScores, weekScores, dayScores, levelPlayed);
                ((LevelSelect)parent).setScores();
                parent.setVisible(true);
                this.setVisible(false);
            }
        }
    }

    private void movePacman() {
        if (!paused) {
            if (canPacmanMove) {
                GamePanel.getPacman().move();
                checkDeath();
                checkWin();
                canPacmanMove = false;
                pacmanTimer.start();
            }
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        GamePanel = new biteme.Maze();
        PauseButton = new javax.swing.JLabel();
        ScoreButton = new javax.swing.JLabel();
        Main_menu_button = new javax.swing.JButton();
        Restart = new javax.swing.JButton();
        helpButton = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setResizable(false);
        addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                formKeyPressed(evt);
            }
        });

        GamePanel.setBackground(new java.awt.Color(0, 0, 0));
        GamePanel.setPreferredSize(new java.awt.Dimension(960, 660));
        GamePanel.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                GamePanelKeyPressed(evt);
            }
        });

        javax.swing.GroupLayout GamePanelLayout = new javax.swing.GroupLayout(GamePanel);
        GamePanel.setLayout(GamePanelLayout);
        GamePanelLayout.setHorizontalGroup(
            GamePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 960, Short.MAX_VALUE)
        );
        GamePanelLayout.setVerticalGroup(
            GamePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 660, Short.MAX_VALUE)
        );

        PauseButton.setFont(new java.awt.Font("Tahoma", 0, 14)); // NOI18N
        PauseButton.setText(Messages.GAME_START);

        ScoreButton.setFont(new java.awt.Font("Tahoma", 0, 14));
        ScoreButton.setText(Messages.GAME_SCORE1 + score + Messages.GAME_SCORE2);

        Main_menu_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/biteme/level_select.png"))); // NOI18N
        Main_menu_button.setMaximumSize(new java.awt.Dimension(150, 20));
        Main_menu_button.setMinimumSize(new java.awt.Dimension(150, 20));
        Main_menu_button.setPreferredSize(new java.awt.Dimension(150, 20));
        Main_menu_button.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                Main_menu_buttonMousePressed(evt);
            }
        });

        Restart.setIcon(new javax.swing.ImageIcon(getClass().getResource("/biteme/restart.png"))); // NOI18N
        Restart.setMaximumSize(new java.awt.Dimension(150, 20));
        Restart.setMinimumSize(new java.awt.Dimension(150, 20));
        Restart.setPreferredSize(new java.awt.Dimension(150, 20));
        Restart.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                Restart(evt);
            }
        });

        helpButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/biteme/help.png"))); // NOI18N
        helpButton.setMaximumSize(new java.awt.Dimension(150, 20));
        helpButton.setMinimumSize(new java.awt.Dimension(150, 20));
        helpButton.setPreferredSize(new java.awt.Dimension(150, 20));
        helpButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                helpButtonMousePressed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(GamePanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                    .addComponent(helpButton, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(Main_menu_button, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(Restart, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 532, Short.MAX_VALUE)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(ScoreButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(PauseButton, javax.swing.GroupLayout.DEFAULT_SIZE, 258, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(GamePanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(42, 42, 42)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(Main_menu_button, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(Restart, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(PauseButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(ScoreButton)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(helpButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(58, 58, 58))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void formKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_formKeyPressed
        // TODO add your handling code here:
    }//GEN-LAST:event_formKeyPressed

    private void GamePanelKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_GamePanelKeyPressed

        PacmanEntity p = this.GamePanel.getPacman();

        if (evt.getKeyCode() == java.awt.event.KeyEvent.VK_ENTER) {
            if (paused) {
                unPause();
                PauseButton.setText(Messages.GAME_UNPAUSED);                
            } else {
                pause();
                score ++;
                ScoreButton.setText(Messages.GAME_SCORE1 + score + Messages.GAME_SCORE2);
                PauseButton.setText(Messages.GAME_PAUSED);
            }
        }

        if (!paused) {
            switch (evt.getKeyCode()) {
                case java.awt.event.KeyEvent.VK_LEFT:
                    p.direction = Direction.LEFT;
                    break;
                case java.awt.event.KeyEvent.VK_RIGHT:
                    p.direction = Direction.RIGHT;
                    break;
                case java.awt.event.KeyEvent.VK_UP:
                    p.direction = Direction.UP;
                    break;
                case java.awt.event.KeyEvent.VK_DOWN:
                    p.direction = Direction.DOWN;
                    break;
            }
        }

        movePacman();
    }//GEN-LAST:event_GamePanelKeyPressed

    private void Main_menu_buttonMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_Main_menu_buttonMousePressed
        parent.setVisible(true);
        this.setVisible(false);
        pause();
    }//GEN-LAST:event_Main_menu_buttonMousePressed

    private void Restart(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_Restart
        Sound.playPacmanDying();
        pause(false);
        GameGUI gameGUI = new GameGUI(parent, levelPlayed);
        gameGUI.setVisible(true);
        this.setVisible(false);
        dispose();

    }//GEN-LAST:event_Restart

    private void helpButtonMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_helpButtonMousePressed
        // Opens help popup
        if (!paused){
            pause(true);
        }
        Popup.help(0);
        this.GamePanel.requestFocus();

    }//GEN-LAST:event_helpButtonMousePressed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    public static biteme.Maze GamePanel;
    private javax.swing.JButton Main_menu_button;
    private javax.swing.JLabel PauseButton;
    private javax.swing.JButton Restart;
    private javax.swing.JLabel ScoreButton;
    private javax.swing.JButton helpButton;
    // End of variables declaration//GEN-END:variables
}
