package biteme;

/**
 * Tile representing a one-way tile.
 * @author N.F.W. Thissen, A.M. Zantema
 * @version 0.0.1
 */
public class OneWayTile extends Tile {

    /**
     *
     * @param m
     */
    public OneWayTile(Maze m, int x, int y) {
        super(m, x, y);

    }

    /**
     * This function gives the direction of the one-way tile.
     * @return The direction of the one-way tile.
     */
    public Direction getDirection() {
        Direction d;
        if (this.getStatus().containsOneWayDown()) {
            d = Direction.DOWN;
        } else if (this.getStatus().containsOneWayUp()) {
            d = Direction.UP;
        } else if (this.getStatus().containsOneWayLeft()) {
            d = Direction.LEFT;
        } else if (this.getStatus().containsOneWayRight()) {
            d = Direction.RIGHT;
        } else {
            d = Direction.NONE;
        }
        return d;
    }

    /**
     * Checks whether an entity can enter this OneWayTile.
     * @param d The direction of the entity that wants to enter.
     * @param e The entity to check for.
     * @return If the entity does NOT come from the direction of the OneWayTile,
     * the entity can enter.
     */
    public boolean canEnter(Direction d, Entity e) {
        Direction opposite = Direction.getOpposite(d);
        Direction thisDirection = this.getDirection();

        return opposite != thisDirection;
    }

    /**
     * This method checks whether this tile contains a ghost or not.
     * @return true if there is a ghost, false if there is no ghost.
     */
    public boolean containsGhost() {
        for (SmartGhost g : maze.getSmartGhosts()) {
            if (g.getTile().equals(this)) {
                return true;
            }
        }
        for (StupidGhost g : maze.getStupidGhosts()) {
            if (g.getTile().equals(this)) {
                return true;
            }
        }

        return false;
    }

    /**
     * Checks whether an entity can leave a tile in a certain direction.
     * @param d The direction in which the entity wants to leave the tile.
     * @param e The entity that wants to leave the tile.
     * @return true if the direction of the entity is the same as the direction of the
     * OneWayTile, else false.
     */
    public boolean canLeave(Direction d, Entity e) {
        Direction thisDirection = this.getDirection();
        return thisDirection == d;
    }

    /**
     * Retrieves the first empty tile to which a box can be moved.
     * @param d
     * @return null. A box cannot move on a OneWayTile.
     */
    public Tile getNextBoxTile(Direction d) {
        // a box cannot move on a OneWayTile
        return null;
    }

    /**
     * This method draws the tile.
     */
    public void draw() {
        String spriteName = "OneWay";
        switch (this.getDirection()) {
            case UP:
                spriteName += "U";
                break;
            case DOWN:
                spriteName += "D";
                break;
            case LEFT:
                spriteName += "L";
                break;
            case RIGHT:
                spriteName += "R";
                break;
        }
        if (this.getStatus().containsCoin()) {
            spriteName += "C";
        }
        maze.drawAtXY(spriteName, getLocationX(), getLocationY());
    }
}
