package biteme;

import java.util.LinkedList;
import java.util.Random;

/**
 * Entity representing a ghost that moves towards Pacman.
 * @author NFW Thissen
 * @version 0.0.1
 */
public class SmartGhost extends GhostEntity {

    /** The chance that this SmartGhost takes a random step, between 0% and 100%. */
    private final int randomChance = 20;

    /** The Random object used to determine random events. */
    private Random rnd = new Random();

    /**
     * Constructor.
     * @param m The GameGUI instance.
     */
    public SmartGhost(Maze m) {
        super(m);
    }

    /**
     * Moves this Ghost in the Direction determined by chooseDirection.
     */
    public void move() {
        direction = this.chooseDirection();
        if (direction != Direction.NONE) {
            this.move(direction);
        }
    }

    /**
     * Chooses the next Direction to move in based on a random chance and Dijkstra's algorithm.
     * @return The next Direction to move in.
     */
    public Direction chooseDirection() {
        Direction dir = Direction.NONE;
        if (rnd.nextInt(101) >= randomChance) {
            // choose nextTileToPacman direction
            dir = Direction.fromTiles(this.getTile(), this.getTile().getNextTileToPacman());
        }
        if (dir != Direction.NONE) {
            return dir;
        } else {
            // either random.nextInt < randomChance, or this ghost cannot get to pacman at all.
            // in both cases, just move randomly
            LinkedList<Direction> possibleDirs = this.getPossibleDirections();
            int randomIndex = rnd.nextInt(possibleDirs.size());
            return possibleDirs.get(randomIndex);
        }
    }

    /**
     * Draws this Ghost on to the Maze.
     */
    public void draw() {
        String spriteName = "SmartGhost";
        if (this.getTile().getStatus().containsCoin()){
            spriteName += "C";
        }
        maze.drawAtXY(spriteName,
                this.getTile().getLocation().x,
                this.getTile().getLocation().y);
    }
}
