package biteme;

import java.util.LinkedList;
import java.util.Random;

/**
 * Entity representing a ghost that randomly chooses direction.
 * @author NFW Thissen
 * @version 0.0.1
 */
public class StupidGhost extends GhostEntity {

    /** The Random object used to determine random events. */
    private Random rnd = new Random();

    /** The possible directions this Ghost could move in the previous step. */
    private LinkedList<Direction> oldPossibleDirections;

    /**
     * Constructor.
     * @param m The Maze instance.
     */
    public StupidGhost(Maze m) {
        super(m);
        oldPossibleDirections = new LinkedList<Direction>();
    }

    /**
     * Moves this Ghost in the Direction determined by chooseDirection.
     */
    public void move() {
        // determine when to call chooseDirection...?
        direction = this.chooseDirection();
        this.move(direction);
    }

    /**
     * Chooses a random Direction from a list of possible Directions.
     * @return A random Direction to move in.
     */
    public Direction chooseDirection() {
        LinkedList<Direction> possibleDirs = this.getPossibleDirections();
        Direction dir = null;

        if (possibleDirs.contains(direction)) {
            // 1. If we can move in the same direction as before
            if (possibleDirs.size() == oldPossibleDirections.size()) {
                // and if the number of possible directions stayed the same,
                // then keep going in the same direction
                dir = direction;
            } 
        }
        if (dir != direction) {
            // 2. If not, we choose a random direction except the opposite of the last direction
            Direction opp = Direction.getOpposite(direction);
            if (possibleDirs.contains(opp)) {
                possibleDirs.remove(opp);
            }
            if (possibleDirs.size() > 0) {
                int randomIndex = rnd.nextInt(possibleDirs.size());
                dir = possibleDirs.get(randomIndex);
            } else {
                // there was no other direction, so just go back
                dir = opp;
            }
        }

        oldPossibleDirections = possibleDirs;
        return dir;
    }

    /**
     * Draws this Ghost on to the Maze.
     */
    public void draw() {
        String spriteName = "StupidGhost";
        if (this.getTile().getStatus().containsCoin()){
            spriteName += "C";
        }
        maze.drawAtXY(spriteName,
                      this.getTile().getLocation().x,
                      this.getTile().getLocation().y);
    }
}
