package biteme;

import java.awt.Point;

/**
 * Base class for every Tile.
 * @author N.F.W. Thissen, A.M. Zantema
 * @version 0.0.1
 */
public abstract class Tile {

    protected Point location;
    private Status status;
    protected Tile nextTileToPacman;

    public int dist;
    public boolean isVisited;

    /**
     *
     */
    protected Maze maze;

    /**
     *
     * @param m
     */
    public Tile(Maze m, int x, int y) {
        status = new Status();
        maze = m;
        location = new Point(x,y);
    }

    /**
     * Gives the status of this tile.
     * @return status.
     */
    public Status getStatus() {
        return status;
    }

    /**
     * Sets the status to a new status.
     * @param s The new status.
     */
    public void setStatus(Status s) {
        this.status = s;
    }

    /**
     * Gives the location of this tile.
     * @return The location
     */
    public Point getLocation() {
        return location;
    }

    /**
     * This function gives the X-coordinate of the tile.
     * @return the X-coordinate of this tile.
     */
    public int getLocationX() {
        return location.x;
    }

    /**
     * This function gives the Y-coorinate of the tile.
     * @return the Y-coordinate of this tile.
     */
    public int getLocationY() {
        return location.y;
    }

    /**
     * This function gives the next tile on the shortest path from this tile to Pacman.
     * @return the next tile on the shortest path from this tile to Pacman.
     */
    public Tile getNextTileToPacman() {
        return nextTileToPacman;
    }

    /**
     * This function changes the next tile on the shortest path from here to Pacman.
     * @param t The next tile on the shortest path from this tile to Pacman.
     */
    public void setNextTileToPacman(Tile t) {
        nextTileToPacman=t;
    }

    /**
     * Moves a box in a certain direction. This tile may contain a "pushing"
     * Pacman, or more general, the contents of this tile do not matter but the box on the
     * next tile (if any) is moved to the first free tile.
     * @param d The direction in which the box should be moved.
     */
    public void moveBox(Direction d) {

          if (d != Direction.NONE) {
            // Kijk of op volgende tile een box staat
            Tile nextTile = maze.getNextTile(this, d);
            if (nextTile.getStatus().containsBox()) {
                // zo ja, neem de tile DAAR weer na
                Tile nextNextTile = maze.getNextTile(nextTile, d);
                // en vraag de eerstvolgende vrije Tile op in die richting (waar de doos heen zou kunnen)
                Tile boxTile = nextNextTile.getNextBoxTile(d);

                // Als er zo'n Tile bestaat dan
                if (boxTile != null) {
                    // Verwijder box op de eerste Tile
                    nextTile.getStatus().clearBox();
                    // Plaats box op de laatste Tile
                    boxTile.getStatus().setBox();

                    nextTile.draw();
                    boxTile.draw();
                }
            }
        }
    }

    /**
     * Draws this tile.
     */
    public abstract void draw();

    /**
     * Checks whether an entity can enter this tile.
     * @param d The direction of the entity.
     * @param e The entity.
     * @return true if the entity can enter this tile.
     */
    public abstract boolean canEnter(Direction d, Entity e);

    /**
     * Looks for the first empty tile to which a box can be moved in a certain
     * direction.
     * @param d The direction in which the box is to be moved.
     * @return The first 'free' tile to which a box can bemoved.
     */
    public abstract Tile getNextBoxTile(Direction d);

    /**
     * Chekcs whether this tile contains a ghost.
     * @return True if this tile contains at least one ghost.
     */
    public abstract boolean containsGhost();

    /**
     * Checks whether an entity can leave a tile in a certain direction.
     * @param d The direction in which the entity wants to leave the tile.
     * @param e The entity that wants to leave the tile.
     * @return True if the entity can leave the tile in direction d.
     */
    public abstract boolean canLeave(Direction d, Entity e);
}
